@echo off
:: --------------------------------------------------------------------
:: buildbufr 
::  - Example batch file to build BUFR library and support programs.  
::    (PC-DOS, Intel Visual Fortran f90/95 V8 & 
::     Microsoft C/C++ .NET 2003 compilers)
::
:: Assumes a directory structure:
::
::            bufr/
::              |
::     +--------+---------+
::     |                  |
::   metdb/            extra/
::
:: and that:
:: - we are curently in bufr/
:: - the MetDB BUFR package, including operational tables, is installed 
::   in bufr/metdb/
:: - non-MetDB files are in bufr/extra/
:: - run-time tables will be placed in BUFR_LIBRARY (normally bufr/)
::
:: Usage: > buildbufr [-b bufrdir] [-c] [-h] [-p prefix] [-t]
::
:: Options:
::  -b set BUFR_LIBRARY path temporariliy to <bufrdir> 
::      BUFR tables will be copied here as working versions. 
::      Default: existing environment variable definition,
::      or if this is not defined, then <prefix>\data\bufr\
::      Note the final '\' in this path.
::
::  -c clean build before make (make from scratch)
::      Default: build only things which have changed
::
::  -h print basic usage info and exit.
::
::  -p set a prefix (root directory) for build. Default: HOME
::      <prefix>\bin       - location for binary (executable) files
::      <prefix>\lib       - location for BUFR object library
::      <prefix>\include   - location for f90 include & module files
::      <prefix>\man\man1  - location for 'man page' files
::      <prefix>\data\bufr - default location for BUFR run-time files
::                           (if BUFR_LIBRARY is not set) 
::
::  -t also build & run MetDB simple test programs. Executable files
::      are built in bufr\metdb\. Default: skip test programs.
::      (test_tables and decbufr are always built & run)
::
:: --------------------------------------------------------------------
::
echo.
echo ============ Build script for BUFR package ============
echo                      (PC-DOS)
set TOP=%CD%
if "%HOME%"=="" goto nohome
:: 
:: --- Parse options after setting defaults ---
::
set clean="n"
set test="n"
set PREFIX=%HOME%
::
:loop
if "%1"=="" goto endloop
if not "%1"=="-b" goto opt2
shift
set BUFR_LIBRARY=%1
shift
goto loop
:opt2
if not "%1"=="-c" goto opt3
set clean="y"
shift
goto loop
:opt3
if not "%1"=="-h" goto opt4
echo.
echo Usage: buildbufr [-b bufrdir] [-c] [-h] [-p prefix] [-t]
goto done
:opt4
if not "%1"=="-p" goto opt5
shift
set PREFIX=%1
shift
goto loop
:opt5
if not "%1"=="-t" goto noopt
set test="y"
shift
goto loop
:noopt
shift
goto loop
:endloop
::
if "%BUFR_LIBRARY%"=="" set BUFR_LIBRARY=%PREFIX%\data\bufr\
::
:: --- Set directories ---
:: 
set BINDIR=%PREFIX%\bin
set LIBDIR=%PREFIX%\lib
set INCDIR=%PREFIX%\include
set MANDIR=%PREFIX%\man
set DATDIR=%PREFIX%\data
::
:: --- Create any missing diretories ---
::
if not exist %PREFIX%\NUL      mkdir %PREFIX%
if not exist %BINDIR%\NUL      mkdir %BINDIR%
if not exist %LIBDIR%\NUL      mkdir %LIBDIR%
if not exist %INCDIR%\NUL      mkdir %INCDIR%
if not exist %MANDIR%\NUL      mkdir %MANDIR%
if not exist %MANDIR%\man1\NUL mkdir %MANDIR%\man1
if not exist %DATDIR%\NUL      mkdir %DATDIR%
if not exist %DATDIR%\bufr\NUL mkdir %DATDIR%\bufr
if not exist %BUFR_LIBRARY%NUL mkdir %BUFR_LIBRARY%

::
set BUFRLIB=%LIBDIR%\bufr.lib
::
:: --- Install operational & local BUFR tables ---
::
echo.
echo ** Copying operational BUFR tables to %BUFR_LIBRARY%...
xcopy %TOP%\metdb\TABLEB   %BUFR_LIBRARY%* /D /Y
xcopy %TOP%\metdb\TABLED   %BUFR_LIBRARY%* /D /Y
xcopy %TOP%\metdb\CODEFIG  %BUFR_LIBRARY%* /D /Y
::
echo.
echo ** Copying local BUFR tables to %BUFR_LIBRARY%...
xcopy    %TOP%\extra\TABLEA   %BUFR_LIBRARY%* /D /Y
xcopy    %TOP%\extra\CT001034 %BUFR_LIBRARY%* /D /Y
if exist %TOP%\extra\TABLEB  xcopy %TOP%\extra\TABLEB  %BUFR_LIBRARY%* /D /Y
if exist %TOP%\extra\TABLED  xcopy %TOP%\extra\TABLED  %BUFR_LIBRARY%* /D /Y
if exist %TOP%\extra\CODEFIG xcopy %TOP%\extra\CODEFIG %BUFR_LIBRARY%* /D /Y
::
:: --- Build MetDB kernel BUFR object library ---
::
echo.
cd %TOP%/metdb
xcopy %TOP%\extra\Makefile_BUFRrelease_PC * /D /Y
xcopy %TOP%\extra\Makefile_BUFRtest_PC    * /D /Y
if %clean%=="n" goto buildlib
echo .
echo ** Cleaning BUFR kernel library...
echo.
nmake /f Makefile_BUFRrelease_PC /nologo clean
if exist %BUFRLIB% del /S %BUFRLIB%
echo.
:: --- here for main library build
:buildlib
echo ** Building BUFR kernel object library %BUFRLIB%...
echo.
nmake /f Makefile_BUFRrelease_PC /nologo
echo.
if not exist bufr.lib goto nolib
copy bufr.lib %BUFRLIB%
::
:: --- Optionally build & run simple MetDB encode/decode test programs ---
::
set BUFRDEC_FILE=%TOP%/metdb/TestMessage.bufr
if %test%=="n" goto doextra
echo.
if %clean%=="n" goto buildtest
echo ** Cleaning test files...
echo.
nmake /nologo /f Makefile_BUFRtest_PC clean
echo.
:buildtest
echo ** Testing basic encode/decode...
echo.
nmake /nologo /f Makefile_BUFRtest_PC PREFIX=%PREFIX%
echo.
echo    - Encoding...
echo.
BUFRencode
echo.
if not exist %BUFRDEC_FILE% goto encfail
echo    - Decoding...
BUFRdecode
goto doextra
:encfail
echo *** Encode failed! ***
::
:: --- Build & run extra utility routines & programs ---
::
:doextra
cd %TOP%\extra
if %clean%=="n" goto buildextra
echo.
echo ** Cleaning BUFR extra utilities...
echo.
nmake /nologo /f Makefile_PC PREFIX=%PREFIX% clean
echo.
:buildextra
echo.
echo ** Adding BUFR extra routines to %BUFRLIB%
echo    and making BUFR utility programs in %BINDIR%...
xcopy portability.fi_pc %INCDIR%\portability.fi /D /Y
echo.
nmake /nologo /f Makefile_PC PREFIX=%PREFIX%
echo.
echo ** Testing Table B and Code Table for ERS-1...
echo.
%BINDIR%\test_tables < test_tables.dat
echo.
echo ** Testing decbufr...
%BINDIR%\decbufr %BUFRDEC_FILE% -d
goto done
::
:: --- Install man pages ---
::
:copyman
echo.
echo ** Copying MANDIR files to %MANDIR%\man1...
xcopy %TOP%\extra\*.1 %MANDIR%\man1\* /D /Y
goto done
:: --- Here if HOME not defined
:nohome
echo.
echo *** Environment variable HOME has not been set. HOME must be set to
echo point to your root directory - e.g.
echo  set HOME=C:\mydir
got done
:: --- here if BUFR library not generated
:nolib
echo.
echo *** Failed to build %BUFRLIB% - correct error(s) and re-run ***
:: --- Finished.
:done
cd %TOP%
set BUFRDEC_FILE=
set BUFRLIB=
set clean=
set PREFIX=
set test=
set TOP=
echo.
